#include "AlesWindow.h"
#include "constants.h"

#include "SettingsFile.h"

#include <File.h>
#include <TypeConstants.h>
#include <Screen.h>
#include <Path.h>
#include <FindDirectory.h>
#include <Directory.h>
#include <Alert.h>
#include <posix/string.h>

AlesWindow::AlesWindow(BRect frame, const char *name,
	window_look look, window_feel feel, uint32 flags,
	entry_ref directory)
		: BWindow(frame, name, look, feel, flags)
{
	RestoreState();
	fView = new GraphicView(this->Bounds(), "view", directory);
	this->AddChild(fView);
}

AlesWindow::~AlesWindow()
{
	SaveState();
}

//設定を読み込み
status_t AlesWindow::RestoreState()
{
	SettingsFile setting(kSettingFile, kSettingDir);
	status_t sts = setting.InitCheck();
	if(sts != B_OK) return sts;
	sts = setting.Load();
	
	//ウィンドウ位置を読み込む
	BRect frame;
	sts = setting.FindRect(kPosition, &frame);
	//(new BAlert("", strerror(sts), "OK"))->Go();
	if(sts == B_OK)
	{
		//読み込めたら移動
		this->MoveTo(frame.LeftTop());
		this->ResizeTo(frame.Width(), frame.Height());
		
		//解像度を変えたりしてスクリーンからはみ出したときは左上に持っていく...はず
		//実は確かめていない
		BScreen screen(this);
		BRect screenFrame = screen.Frame();
		if (screenFrame.Contains(frame.LeftTop()) == false)
			this->MoveTo(B_ORIGIN);
	}
	sts = setting.FindRect(kNormalPosition, &frame);
	//(new BAlert("", strerror(sts), "OK"))->Go();
	if(sts == B_OK)
	{
		fNormalFrame = frame;
	}
	else
	{
		fNormalFrame = BRect(100, 100, 400, 400);
	}
	return sts;
	

}

//設定を保存
status_t AlesWindow::SaveState()
{
	SettingsFile setting(kSettingFile, kSettingDir);
	status_t sts = setting.InitCheck();
	if(sts != B_OK)
		return sts;
	sts = setting.Load();
	setting.SetRect(kPosition, this->Frame());
	setting.SetRect(kNormalPosition, fNormalFrame);
	sts = setting.Save();
	return sts;
}

void AlesWindow::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case B_MOUSE_WHEEL_CHANGED:
			fView->MessageReceived(message);
			break;
		case FULL_SCREEN:
			FullScreen();
			break;
		default:
			BWindow::MessageReceived(message);
			break;
	}
	
}

void AlesWindow::FullScreen()
{
	BScreen screen;
	BRect screenRect = screen.Frame();
	/*
	Zoom(B_ORIGIN, screenRect.Width(), screenRect.Height());
	*/
	BRect newFrame;
	if (screenRect == Frame())
	{
		newFrame = fNormalFrame;
	}
	else
	{
		newFrame = screenRect;
		fNormalFrame = Frame();
	}
	MoveTo(newFrame.LeftTop());
	ResizeTo(newFrame.Width(), newFrame.Height());
	
	/*
	MoveTo(point);
	ResizeTo(width, height);
	*/
}
