#include "FileCollector.h"
#include "MFileUtils.h"

#include <algorithm>
#include <vector>

#include <stdio.h>
#include <sys/stat.h>

#include <Directory.h>
#include <Entry.h>

void GetStatAB(const entry_ref *refA, const entry_ref *refB, struct stat *statA, struct stat *statB)
{
	BEntry entryA(refA), entryB(refB);
	entryA.GetStat(statA);
	entryB.GetStat(statB);
	return;
}

//名前でソート
bool CompareByName(entry_ref refA, entry_ref refB)
{
	if (strcmp(refA.name, refB.name) < 0)
		return true;
	else
		return false;
}

//サイズでソート
bool CompareBySize(entry_ref refA, entry_ref refB)
{
	struct stat statA, statB;
	GetStatAB(&refA, &refB,&statA, &statB);
	if (statA.st_size < statB.st_size)
		return true;
	else
		return false;
}

//作成日時でソート
bool CompareByCreationTime(entry_ref refA, entry_ref refB)
{
	struct stat statA, statB;
	GetStatAB(&refA, &refB,&statA, &statB);
	if (statA.st_ctime < statB.st_ctime)
		return true;
	else
		return false;
}

//修正日時でソート
bool CompareByModificationTime(entry_ref refA, entry_ref refB)
{
	struct stat statA, statB;
	GetStatAB(&refA, &refB,&statA, &statB);
	if (statA.st_mtime < statB.st_mtime)
		return true;
	else
		return false;
}

//アクセス日時でソート
bool CompareByAccessTime(entry_ref refA, entry_ref refB)
{
	struct stat statA, statB;
	GetStatAB(&refA, &refB,&statA, &statB);
	if (statA.st_atime < statB.st_atime)
		return true;
	else
		return false;
}

FileCollector::FileCollector(entry_ref dir, const char *type)
{
	fDirectory = dir;
	Collect(type);
	fFirstAccess = true;
	fSortKey = NAME;
	fReverseSort = false;
}

FileCollector::~FileCollector()
{

}

//次のファイル
status_t FileCollector::GetNextRef(entry_ref *ref)
{
	if (fFirstAccess)
	{
		fFirstAccess = false;
		fIterator = fList.begin();
	}
	else
	{
		fIterator++;
	}

	if (fIterator == fList.end())
	{
		fIterator--;
		return B_ERROR;
	}
	*ref = *fIterator;
	return B_OK;
}

//前のファイル
status_t FileCollector::GetPrevRef(entry_ref *ref)
{
	if (fFirstAccess)
	{
		fFirstAccess = false;
		fIterator = fList.end();
	}
	if (fIterator == fList.begin())
		return B_ERROR;
	fIterator--;
	*ref = *fIterator;
	return B_OK;
}

//対象ディレクトリを返す
entry_ref FileCollector::GetDirectory()
{
	return fDirectory;
}

//ファイル種類を増やす
void FileCollector::AddType(const char *type)
{
	Collect(type);
}

//リセットする
void FileCollector::ResetTo(entry_ref dir, const char *type)
{
	fDirectory = dir;
	fList.clear();
	Collect(type);
	Sort();
	fFirstAccess = true;
}

//イテレータの位置をセット
void FileCollector::SetPosition(entry_ref ref)
{
	fIterator = find(fList.begin(), fList.end(), ref);
	fFirstAccess = false;
}

//要素の数を返す
int32 FileCollector::Count()
{
	return fList.size();
}

void FileCollector::SortBy(uint32 key, bool reverse)
{
	fSortKey = key;
	fReverseSort = reverse;
	Sort();
}

void FileCollector::ResetPosition()
{
	fFirstAccess = true;
}

void FileCollector::Sort()
{
	switch (fSortKey)
	{
		case SIZE :
		{
			fList.sort(CompareBySize);
			break;
		}
		case CREATION_TIME :
		{
			fList.sort(CompareByCreationTime);
			break;
		}
		case MODIFICATION_TIME :
		{
			fList.sort(CompareByModificationTime);
			break;
		}
		case ACCESS_TIME :
		{
			fList.sort(CompareByAccessTime);
			break;
		}
		case NAME :
		default :
		{
			fList.sort(CompareByName);
			break;
		}
	}
	if (fReverseSort)
	{
		fList.reverse();
	}
	fFirstAccess = true;

}

//ある種類のファイルを集める
void FileCollector::Collect(const char *type)
{
	entry_ref ref;
	BDirectory dir(&fDirectory);
	while (dir.GetNextRef(&ref) == B_OK)
	{
		if (MFileUtils::IsThisType(&ref, type) == true)
			fList.push_back(ref);
	}

}

