#ifndef GRAPHIC_VIEW_H
#define GRAPHIC_VIEW_H

#include <View.h>
#include <Entry.h>

#include "FileCollector.h"
#include "SettingsFile.h"

enum
{
	ORIGINAL_SIZE = 'mode',
	SCALE
};

enum
{
	NEXT_IMAGE = 'imge',
	PREVIOUS_IMAGE,
	LAST_IMAGE,
	FIRST_IMAGE,
	NEXT_DIRECTORY,
	PREVIOUS_DIRECTORY,
	PARENT_DIRECTORY,
	CHILD_DIRECTORY
};

const uint32 HELP = 'help';
const uint32 FULL_SCREEN = 'full';

const char kPopUpMenu[] = "PopUpMenu";
const char kMode[] = "Mode";
const char kOriginalSize[] = "Original Size";
const char kScale[] = "Scale";
const char kSortBy[] = "Sort By";
const char kName[] = "Name";
const char kSize[] = "Size";
const char kCreationTime[] = "Creation Time";
const char kModificationTime[] = "Modification Time";
const char kAccessTime[] = "Access Time";
const char kHelp[] = "Help";
const char kQuit[] = "Quit";

class GraphicView : public BView
{
public :
	GraphicView(BRect frame, const char *name, entry_ref initialDirectory);
	~GraphicView();
	virtual void Draw(BRect updateRect);
	virtual void KeyUp(const char *bytes, int32 numBytes);
	virtual void KeyDown(const char *bytes, int32 numBytes);
	virtual void Pulse();
	virtual void MessageReceived(BMessage *message);
	virtual void AttachedToWindow();
	virtual void FrameResized(float width, float height);
	virtual void MouseDown(BPoint point);
private :
	status_t NextImage();
	status_t PreviousImage();
	status_t FirstImage();
	status_t LastImage();
	void ShowImage(entry_ref ref);
	void PrintNoImage();
	status_t ShowNextDirectory();
	status_t ShowPreviousDirectory();
	status_t ChildDirectory();
	status_t ParentDirectory();
	status_t ShowChildDirectory();
	status_t ShowParentDirectory();
	status_t ChangeDirectory(int32 moveBy);
	void Reset(entry_ref dirRef);
	void CreateMenu();
	
	void MouseWheelChanged(BMessage *message);
	
	status_t RestoreState();
	status_t SaveState();
	void SaveMarked(SettingsFile *setting, const char *label, const char *save_name);
	BMenuItem *RestoreMarked(SettingsFile *setting, const char *label, const char *save_name);
	
	FileCollector *fCollector;
	BBitmap *fBitmap;
	BPopUpMenu *fMenu;
	uint32 fMode;
	bool fKeyDown;
	entry_ref fRef;
};

#endif //GRAPHIC_VIEW_H